__version__ = "0.2.3"
__license__ = "GPLv3"

__author__ = "José Antonio Perdiguero López"
__email__ = "perdy.hh@gmail.com"

__url__ = "https://github.com/PeRDy/apistar-crud"
__description__ = "API Star tools to create CRUD resources."
