<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>{% block title %}Admin{% endblock title %}</title>
  <link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Roboto:300,400,500">
  <link rel="stylesheet" href="https://fonts.googleapis.com/icon?family=Material+Icons">
  <link rel="stylesheet" href="/static/apistar_crud/styles.react.css">
</head>
<body>
  <div id="root"></div>
</body>
<script src="/static/apistar_crud/runtime~index.js"></script>
<script src="/static/apistar_crud/styles.react.js"></script>
<script src="/static/apistar_crud/vendors~index.js"></script>
<script src="/static/apistar_crud/index.js"></script>
</html>
